@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2024 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -21,4 +21,8 @@
     PO Box 1080 Blindern, NO-0316 Oslo, Norway
 */
 
-auto algo_run(struct Parameters const & parameters) -> void;
+#include <vector>
+
+
+auto algo_run(struct Parameters const & parameters,
+              std::vector<struct seqinfo_s> & seqindex_v) -> void;
