@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2024 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -23,16 +23,17 @@
 
 #include <cstdio>  // std::FILE
 #include <cstdint>  // uint64_t
+#include <vector>
 
 
-auto db_read(const char * filename, struct Parameters const & parameters) -> void;
+auto db_read(struct Parameters const & parameters,
+             std::vector<char> & data_v,
+             std::vector<struct seqinfo_s> & seqindex_v,
+             std::vector<uint64_t> & zobrist_tab_base_v,
+             std::vector<uint64_t> & zobrist_tab_byte_base_v) -> void;
 
 auto db_getsequencecount() -> unsigned int;
 
-auto db_getnucleotidecount() -> uint64_t;
-
-auto db_getlongestheader() -> unsigned int;
-
 auto db_getlongestsequence() -> unsigned int;
 
 auto db_getseqinfo(uint64_t seqno) -> struct seqinfo_s *;
@@ -43,9 +44,9 @@ auto db_getsequencelen(uint64_t seqno) -> unsigned int;
 
 auto db_gethash(uint64_t seqno) -> uint64_t;
 
-void db_getsequenceandlength(uint64_t seqno,
-                             char ** address,
-                             unsigned int * length);
+auto db_getsequenceandlength(uint64_t seqno,
+                             char * & address,
+                             unsigned int & length) -> void;
 
 auto db_getheader(uint64_t seqno) -> char *;
 
@@ -53,15 +54,12 @@ auto db_getheaderlen(uint64_t seqno) -> unsigned int;
 
 auto db_getabundance(uint64_t seqno) -> uint64_t;
 
-void db_showall();
+auto db_free() -> void;
 
-void db_free();
+auto db_qgrams_init(struct Parameters const & parameters,
+                    std::vector<struct seqinfo_s> & seqindex_v) -> void;
 
-void db_putseq(int64_t seqno);
-
-void db_qgrams_init();
-
-void db_qgrams_done();
+auto db_qgrams_done() -> void;
 
 auto db_fprintseq(std::FILE * fastaout_fp, unsigned int seqno) -> void;
 
