@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2024 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -23,23 +23,20 @@
 
 #include <array>
 #include <cstdint>  // int64_t
+#include <vector>
 
 
 // refactor: 'n_cells' is already defined in 'score_matrix.h'
 constexpr auto n_cells_ = 32ULL;  // number of chars in sym_nt
 
-void nw(char * dseq,
-        int64_t dlen,
+auto nw(char * dseq,
+        uint64_t dlen,
         char * qseq,
-        int64_t qlen,
+        uint64_t qlen,
         const std::array<int64_t, n_cells_ * n_cells_> & score_matrix,
-        int64_t gapopen,
-        int64_t gapextend,
-        int64_t * nwscore,
-        int64_t * nwdiff,
-        int64_t * nwalignmentlength,
-        char ** nwalignment,
-        unsigned char * dir,
-        int64_t * hearray,
-        uint64_t queryno,
-        uint64_t dbseqno);
+        uint64_t gapopen,
+        uint64_t gapextend,
+        uint64_t & nwdiff,
+        std::vector<unsigned char> & directions,
+        std::vector<uint64_t> & hearray,
+        std::vector<char> & raw_alignment) -> void;
