@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2024 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -21,15 +21,17 @@
     PO Box 1080 Blindern, NO-0316 Oslo, Norway
 */
 
-#include <cstdint>
+#include <cstdint>  // uint64_t
+#include <vector>
 
 
-void findqgrams(unsigned char * seq, uint64_t seqlen,
-                unsigned char * qgramvector);
+auto findqgrams(unsigned char * seq, uint64_t seqlen,
+                unsigned char * qgramvector) -> void;
 auto qgram_diff(uint64_t seqno_a, uint64_t seqno_b) -> uint64_t;
-void qgram_diff_fast(uint64_t seed,
+auto qgram_diff_fast(uint64_t seed,
                      uint64_t listlen,
                      uint64_t * amplist,
-                     uint64_t * difflist);
-void qgram_diff_init();
-void qgram_diff_done();
+                     uint64_t * difflist,
+                     std::vector<struct thread_info_s>& thread_info_v) -> void;
+auto qgram_diff_init(std::vector<struct thread_info_s>& thread_info_v) -> void;
+auto qgram_diff_done() -> void;
