@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2024 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -21,15 +21,20 @@
     PO Box 1080 Blindern, NO-0316 Oslo, Norway
 */
 
-#include <cinttypes>
+#include "utils/threads.h"
+#include <cstdint>  // int64_t
+#include <vector>
 
-void search_all(uint64_t query_no);
-void search_do(uint64_t query_no,
+
+auto search_all(uint64_t query_no) -> void;
+auto search_do(uint64_t query_no,
                uint64_t listlength,
                uint64_t * targets,
                uint64_t * scores,
                uint64_t * diffs,
                uint64_t * alignlengths,
-               int bits);
-void search_begin();
-void search_end();
+               int bits,
+               ThreadRunner * search_threads) -> void;
+auto search_begin(std::vector<struct Search_data>& search_data_v) -> void;
+auto search_end() -> void;
+auto search_worker_core(int64_t thread_id) -> void;
