@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2024 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -22,14 +22,16 @@
 */
 
 #include <cstdint>
+#include <vector>
+
 
 using WORD = unsigned short;
 
-void search16(WORD * * q_start,
+auto search16(std::vector<WORD *> & q_start,
               WORD gap_open_penalty,
               WORD gap_extend_penalty,
               WORD * score_matrix,
-              WORD * dprofile,
+              std::vector<WORD> & dprofile,
               WORD * hearray,
               uint64_t sequences,
               uint64_t * seqnos,
@@ -37,6 +39,4 @@ void search16(WORD * * q_start,
               uint64_t * diffs,
               uint64_t * alignmentlengths,
               uint64_t qlen,
-              uint64_t dirbuffersize,
-              uint64_t * dirbuffer,
-              uint64_t longestdbsequence);
+              std::vector<uint64_t> & dirbuffer) -> void;
