@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2024 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -22,14 +22,16 @@
 */
 
 #include <cstdint>
+#include <vector>
+
 
 using BYTE = unsigned char;
 
-void search8(BYTE * * q_start,
+auto search8(std::vector<BYTE *> & q_start,
              BYTE gap_open_penalty,
              BYTE gap_extend_penalty,
              BYTE * score_matrix,
-             BYTE * dprofile,
+             std::vector<BYTE> & dprofile,
              BYTE * hearray,
              uint64_t sequences,
              uint64_t * seqnos,
@@ -37,6 +39,4 @@ void search8(BYTE * * q_start,
              uint64_t * diffs,
              uint64_t * alignmentlengths,
              uint64_t qlen,
-             uint64_t dirbuffersize,
-             uint64_t * dirbuffer,
-             uint64_t longestdbsequence);
+             std::vector<uint64_t> & dirbuffer) -> void;
