@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2024 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -38,13 +38,10 @@
 #include <cstdint>
 #include <smmintrin.h>
 
-#endif
-#endif
 
 using VECTORTYPE = __m128i;
 
-
-void align_cells_regular_16_sse41(VECTORTYPE * Sm,
+auto align_cells_regular_16_sse41(VECTORTYPE * Sm,
                                   VECTORTYPE * hep,
                                   VECTORTYPE ** qp,
                                   VECTORTYPE * Qm,
@@ -52,9 +49,9 @@ void align_cells_regular_16_sse41(VECTORTYPE * Sm,
                                   uint64_t ql,
                                   VECTORTYPE * F0,
                                   uint64_t * dir_long,
-                                  VECTORTYPE * H0);
+                                  VECTORTYPE * H0) -> void;
 
-void align_cells_masked_16_sse41(VECTORTYPE * Sm,
+auto align_cells_masked_16_sse41(VECTORTYPE * Sm,
                                  VECTORTYPE * hep,
                                  VECTORTYPE ** qp,
                                  VECTORTYPE * Qm,
@@ -66,4 +63,7 @@ void align_cells_masked_16_sse41(VECTORTYPE * Sm,
                                  VECTORTYPE * Mm,
                                  VECTORTYPE * MQ,
                                  VECTORTYPE * MR,
-                                 VECTORTYPE * MQ0);
+                                 VECTORTYPE * MQ0) -> void;
+
+#endif
+#endif
