@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2024 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -21,13 +21,19 @@
     PO Box 1080 Blindern, NO-0316 Oslo, Norway
 */
 
+#ifdef __x86_64__
+#ifdef __SSE3__
+
 using WORD = unsigned short;
 using BYTE = unsigned char;
 
-void dprofile_shuffle8(BYTE * dprofile,
+auto dprofile_shuffle8(BYTE * dprofile,
                        BYTE * score_matrix,
-                       BYTE * dseq_byte);
+                       BYTE * dseq_byte) -> void;
 
-void dprofile_shuffle16(WORD * dprofile,
+auto dprofile_shuffle16(WORD * dprofile,
                         WORD * score_matrix,
-                        BYTE * dseq_byte);
+                        BYTE * dseq_byte) -> void;
+
+#endif
+#endif
