@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2024 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -26,21 +26,31 @@
 #include "algod1.h"
 #include "db.h"
 #include "derep.h"
+#include "utils/alignment_parameters.h"
 #include "utils/fatal.h"
 #include "utils/gcd.h"
-#include "util.h"
-#include "x86_cpu_features.h"
+#include "utils/input_output.h"
+#include "utils/open_and_close_files.h"
+#include "utils/opt_boundary.h"
+#include "utils/opt_log.h"
+#include "utils/opt_no_cluster_breaking.h"
+#include "utils/opt_threads.h"
+#include "utils/seqinfo.h"
+#include "utils/x86_cpu_features.h"
+#include "zobrist.h"
 #include <algorithm>  // std::min()
 #include <array>
+#include <cassert>
 #include <cinttypes>  // macros PRIu64 and PRId64
-#include <climits>
 #include <cstdint>  // int64_t
 #include <cstdio>  // FILE, fclose, stderr  // refactoring: replace with <fstream>
 #include <cstdlib>  // std::exit
 #include <getopt.h>  // getopt_long, optarg, optind, opterr, struct
                      // option (no_argument, required_argument)
-
+#include <iterator>  // std::next
+#include <limits>
 #include <string>
+// #include <unistd.h>  // getopt_long... FAIL
 #include <vector>
 
 #ifndef PRId64
@@ -52,15 +62,18 @@ constexpr char PRId64[] = "ld";
 #endif
 
 
-static_assert(INT_MAX > INT16_MAX, "Your compiler uses very short integers.");
+constexpr auto int_max = std::numeric_limits<int>::max();
+constexpr auto int16_max = std::numeric_limits<int16_t>::max();
+static_assert(int_max > int16_max, "Your compiler uses very short integers.");
+const std::string swarm_version {"3.1.5"};
+
 
 /* OPTIONS */
 
-struct Parameters parameters;
 std::string opt_log;
 
 int64_t opt_boundary;
-bool opt_no_otu_breaking {false};
+bool opt_no_cluster_breaking {false};
 int64_t opt_threads;
 
 int64_t penalty_mismatch;
@@ -70,21 +83,11 @@ int64_t penalty_gapopen;
 
 /* fine names and command line options */
 
-std::FILE * outfile {nullptr};
-std::FILE * statsfile {nullptr};
-std::FILE * uclustfile {nullptr};
-std::FILE * logfile {stderr};  // cstdio stderr macro is expanded to type std::FILE*
-std::FILE * internal_structure_file {nullptr};
-std::FILE * fp_seeds {nullptr};
-std::FILE * network_file {nullptr};
-
 constexpr int n_options {26};
-std::array<bool, n_options> used_options {{}};  // value initialization sets values to 'false'
-
-const std::string short_options = "a:b:c:d:e:fg:hi:j:l:m:no:p:rs:t:u:vw:xy:z"; /* unused: kq */
 
+// refactoring: add option -q (no-cluster-breaking)
 const std::array<struct option, 25> long_options = {
-  {
+  { // struct option { name, has_arg, flag, val }
    {"append-abundance",      required_argument, nullptr, 'a' },
    {"boundary",              required_argument, nullptr, 'b' },
    {"ceiling",               required_argument, nullptr, 'c' },
@@ -117,7 +120,7 @@ const std::array<struct option, 25> long_options = {
 const std::vector<std::string> header_message
   {"Swarm ", swarm_version,
    "\n",
-   "Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe\n",
+   "Copyright (C) 2012-2024 Torbjorn Rognes and Frederic Mahe\n",
    "https://github.com/torognes/swarm\n",
    "\n",
    "Mahe F, Rognes T, Quince C, de Vargas C, Dunthorn M (2014)\n",
@@ -157,7 +160,7 @@ const std::vector<std::string> args_usage_message
    "Input/output options:\n",
    " -a, --append-abundance INTEGER      value to use when abundance is missing\n",
    " -i, --internal-structure FILENAME   write internal cluster structure to file\n",
-   " -j, --network_file FILENAME         dump sequence network to file\n",
+   " -j, --network-file FILENAME         dump sequence network to file\n",
    " -l, --log FILENAME                  log to file, not to stderr\n",
    " -o, --output-file FILENAME          output result to file (stdout)\n",
    " -r, --mothur                        output using mothur-like format\n",
@@ -184,7 +187,7 @@ auto args_long(char * str, const char * option) -> int64_t
 {
   static constexpr int base_value {10};
   char * endptr {nullptr};
-  const int64_t temp = strtol(str, & endptr, base_value);
+  const int64_t number = strtol(str, & endptr, base_value);
   if (*endptr != 0)
     {
       fatal(error_prefix, "Invalid numeric argument for option ", option, ".\n\n",
@@ -194,77 +197,76 @@ auto args_long(char * str, const char * option) -> int64_t
             "   (swarm accepts '--help' or '-h', but not '-help')\n\n",
             "Please see 'swarm --help' for more details.");
     }
-  return temp;
+  return number;
 }
 
 
-auto args_show() -> void  // refactoring: pass a ref to parameters as argument
+auto args_show(struct Parameters const & parameters) -> void
 {
 #ifdef __x86_64__
-  cpu_features_detect(parameters);
-  cpu_features_test(parameters);
   cpu_features_show(parameters);
 #endif
 
-  std::fprintf(logfile, "Database file:     %s\n", parameters.input_filename.c_str());
-  std::fprintf(logfile, "Output file:       %s\n", parameters.opt_output_file.c_str());
+  std::fprintf(parameters.logfile, "Database file:     %s\n", parameters.input_filename.c_str());
+  std::fprintf(parameters.logfile, "Output file:       %s\n", parameters.opt_output_file.c_str());
   if (not parameters.opt_statistics_file.empty()) {
-    std::fprintf(logfile, "Statistics file:   %s\n", parameters.opt_statistics_file.c_str());
+    std::fprintf(parameters.logfile, "Statistics file:   %s\n", parameters.opt_statistics_file.c_str());
   }
   if (not parameters.opt_uclust_file.empty()) {
-    std::fprintf(logfile, "Uclust file:       %s\n", parameters.opt_uclust_file.c_str());
+    std::fprintf(parameters.logfile, "Uclust file:       %s\n", parameters.opt_uclust_file.c_str());
   }
   if (not parameters.opt_internal_structure.empty()) {
-    std::fprintf(logfile, "Int. struct. file  %s\n", parameters.opt_internal_structure.c_str());
+    std::fprintf(parameters.logfile, "Int. struct. file  %s\n", parameters.opt_internal_structure.c_str());
   }
   if (not parameters.opt_network_file.empty()) {
-    std::fprintf(logfile, "Network file       %s\n", parameters.opt_network_file.c_str());
+    std::fprintf(parameters.logfile, "Network file       %s\n", parameters.opt_network_file.c_str());
   }
-  std::fprintf(logfile, "Resolution (d):    %" PRId64 "\n", parameters.opt_differences);
-  std::fprintf(logfile, "Threads:           %" PRId64 "\n", opt_threads);
+  std::fprintf(parameters.logfile, "Resolution (d):    %" PRId64 "\n", parameters.opt_differences);
+  std::fprintf(parameters.logfile, "Threads:           %" PRId64 "\n", parameters.opt_threads);
 
   if (parameters.opt_differences > 1)
     {
-      std::fprintf(logfile,
+      std::fprintf(parameters.logfile,
               "Scores:            match: %" PRId64 ", mismatch: %" PRId64 "\n",
               parameters.opt_match_reward, parameters.opt_mismatch_penalty);
-      std::fprintf(logfile,
+      std::fprintf(parameters.logfile,
               "Gap penalties:     opening: %" PRId64 ", extension: %" PRId64 "\n",
               parameters.opt_gap_opening_penalty, parameters.opt_gap_extension_penalty);
-      std::fprintf(logfile,
+      std::fprintf(parameters.logfile,
               "Converted costs:   mismatch: %" PRId64 ", gap opening: %" PRId64 ", "
               "gap extension: %" PRId64 "\n",
-              parameters.penalty_mismatch, penalty_gapopen, penalty_gapextend);
+              parameters.penalty_mismatch, parameters.penalty_gapopen, parameters.penalty_gapextend);
     }
-  std::fprintf(logfile, "Break clusters:        %s\n",
-          opt_no_otu_breaking ? "No" : "Yes");
+  std::fprintf(parameters.logfile, "Break clusters:    %s\n",
+          parameters.opt_no_cluster_breaking ? "No" : "Yes");
   if (parameters.opt_fastidious) {
-    std::fprintf(logfile, "Fastidious:        Yes, with boundary %" PRId64 "\n",
-            opt_boundary);
+    std::fprintf(parameters.logfile, "Fastidious:        Yes, with boundary %" PRId64 "\n",
+            parameters.opt_boundary);
   }
   else {
-    std::fprintf(logfile, "Fastidious:        No\n");
+    std::fprintf(parameters.logfile, "Fastidious:        No\n");
   }
-  std::fprintf(logfile, "\n");
+  std::fprintf(parameters.logfile, "\n");
 }
 
 
-void show(const std::vector<std::string> & message)
+auto show(const std::vector<std::string> &message,
+          std::FILE * log_stream) -> void
 {
   for (const auto & message_element : message) {
-    std::fprintf(logfile, "%s", message_element.c_str());
+    std::fprintf(log_stream, "%s", message_element.c_str());
   }
 }
 
 
-void args_init(int argc, char **argv, std::array<bool, n_options> & used_options)
+auto args_init(int argc, char **argv, struct Parameters & parameters) -> std::array<bool, n_options>
 {
   /* Set defaults */
-  static constexpr unsigned int boundary_default {3};
-  static constexpr unsigned int threads_default {1};
+  static const std::string short_options = "a:b:c:d:e:fg:hi:j:l:m:no:p:rs:t:u:vw:xy:z"; /* unused: kq */
+  std::array<bool, n_options> used_options {{}};  // value initialization sets values to 'false'
 
-  opt_boundary = boundary_default;
-  opt_threads = threads_default;
+  opt_boundary = parameters.opt_boundary;
+  opt_threads = parameters.opt_threads;
   opterr = 1;  // unused variable? get_opt option?
 
   int option_character {0};
@@ -282,17 +284,24 @@ void args_init(int argc, char **argv, std::array<bool, n_options> & used_options
 
     if ((option_character >= 'a') and (option_character <= 'z'))
       {
-        auto optindex = static_cast<unsigned int>(option_character - 'a');  // c - 'a' cannot be negative
+        assert(option_character - 'a' >= 0);
+        assert(option_character <= std::numeric_limits<char>::max());
+        auto optindex = static_cast<unsigned int>(option_character - 'a');
         if (used_options[optindex])
           {
-            int longoptindex {0};
-            while (long_options[longoptindex].name != nullptr)
-              {
-                if (long_options[longoptindex].val == option_character) {
-                  break;
-                }
-                ++longoptindex;
+            auto longoptindex = 0UL;
+            for (const auto& long_option: long_options) {
+              assert(long_option.name != nullptr);
+              if (long_option.name == nullptr) {
+                break; // refactoring: unreachable?
               }
+
+              if (long_option.val == option_character) {
+                break;
+              }
+              ++longoptindex;
+            }
+
             fatal(error_prefix, "Option -", static_cast<char>(option_character),
                   " or --", long_options[longoptindex].name,
                   " specified more than once.");
@@ -309,7 +318,8 @@ void args_init(int argc, char **argv, std::array<bool, n_options> & used_options
 
       case 'b':
         /* boundary */
-        opt_boundary = args_long(optarg, "-b or --boundary");
+        parameters.opt_boundary = args_long(optarg, "-b or --boundary");
+        opt_boundary = parameters.opt_boundary;
         break;
 
       case 'c':
@@ -354,7 +364,8 @@ void args_init(int argc, char **argv, std::array<bool, n_options> & used_options
 
       case 'l':
         /* log */
-        opt_log = optarg;
+        parameters.opt_log = optarg;
+        opt_log = parameters.opt_log;
         break;
 
       case 'm':
@@ -363,8 +374,9 @@ void args_init(int argc, char **argv, std::array<bool, n_options> & used_options
         break;
 
       case 'n':
-        /* no-otu-breaking */
-        opt_no_otu_breaking = true;
+        /* no-cluster-breaking */
+        parameters.opt_no_cluster_breaking = true;
+        opt_no_cluster_breaking = parameters.opt_no_cluster_breaking;
         break;
 
       case 'o':
@@ -389,7 +401,8 @@ void args_init(int argc, char **argv, std::array<bool, n_options> & used_options
 
       case 't':
         /* threads */
-        opt_threads = args_long(optarg, "-t or --threads");
+        parameters.opt_threads = args_long(optarg, "-t or --threads");
+        opt_threads = parameters.opt_threads;
         break;
 
       case 'u':
@@ -423,36 +436,51 @@ void args_init(int argc, char **argv, std::array<bool, n_options> & used_options
         break;
 
       default:
-        show(header_message);
-        show(args_usage_message);
+        show(header_message, parameters.logfile);
+        show(args_usage_message, parameters.logfile);
         fatal();
     }
   }
 
   if (optind < argc) {  // external variable defined in unistd.h for
                         // use with the getopt function
-    parameters.input_filename = argv[optind];
+    parameters.input_filename = *std::next(argv, optind);
   }
 
   // scoring system
   parameters.penalty_mismatch = 2 * parameters.opt_match_reward + 2 * parameters.opt_mismatch_penalty;
-  penalty_gapopen = 2 * parameters.opt_gap_opening_penalty;
-  penalty_gapextend = parameters.opt_match_reward + 2 * parameters.opt_gap_extension_penalty;
+  penalty_mismatch = parameters.penalty_mismatch;
+  parameters.penalty_gapopen = 2 * parameters.opt_gap_opening_penalty;
+  penalty_gapopen = parameters.penalty_gapopen;
+  parameters.penalty_gapextend = parameters.opt_match_reward + 2 * parameters.opt_gap_extension_penalty;
+  penalty_gapextend = parameters.penalty_gapextend;
 
-  const int64_t penalty_factor {gcd(gcd(parameters.penalty_mismatch, penalty_gapopen), penalty_gapextend)};
+  const int64_t penalty_factor {gcd(gcd(parameters.penalty_mismatch, parameters.penalty_gapopen), parameters.penalty_gapextend)};
 
   // clang: risk of DivideZero, but that would require gcd(0, 0) which is not possible
   parameters.penalty_mismatch /= penalty_factor;
   penalty_mismatch = parameters.penalty_mismatch;
-  penalty_gapopen /= penalty_factor;
-  penalty_gapextend /= penalty_factor;
+  parameters.penalty_gapopen /= penalty_factor;
+  penalty_gapopen = parameters.penalty_gapopen;
+  parameters.penalty_gapextend /= penalty_factor;
+  penalty_gapextend = parameters.penalty_gapextend;
+
+#ifdef __x86_64__
+  cpu_features_detect(parameters);
+  cpu_features_test(parameters);
+#endif
+
+  return used_options;
 }
 
 
-void args_check(std::array<bool, n_options> & used_options) {
+auto args_check(const std::array<bool, n_options> &used_options,
+                struct Parameters const & parameters) -> void {
+  static constexpr auto uint8_max = std::numeric_limits<uint8_t>::max();
+  static constexpr auto uint16_max = std::numeric_limits<uint16_t>::max();
   static constexpr unsigned int min_bits_per_entry {2};
   static constexpr unsigned int max_bits_per_entry {64};
-  static constexpr unsigned int min_ceiling {8};
+  static constexpr unsigned int min_ceiling {40};
   static constexpr unsigned int max_ceiling {1U << 30U};  // 1,073,741,824 (MiB of RAM)
   static constexpr unsigned int max_threads {512};
   // meaning of the used_options values
@@ -465,15 +493,15 @@ void args_check(std::array<bool, n_options> & used_options) {
   static constexpr unsigned int mismatch_penalty_index {15};
   static constexpr unsigned int bloom_bits_index {24};
 
-  if ((opt_threads < 1) or (opt_threads > max_threads))
+  if ((parameters.opt_threads < 1) or (parameters.opt_threads > max_threads))
     {
       fatal(error_prefix, "Illegal number of threads specified with "
             "-t or --threads, must be in the range 1 to ", max_threads, ".");
     }
 
-  if ((parameters.opt_differences < 0) or (parameters.opt_differences > UINT8_MAX)) {
+  if ((parameters.opt_differences < 0) or (parameters.opt_differences > uint8_max)) {
     fatal(error_prefix, "Illegal number of differences specified with -d or --differences, "
-          "must be in the range 0 to ", UINT8_MAX, ".");
+          "must be in the range 0 to ", uint8_max, ".");
   }
 
   if (parameters.opt_fastidious and (parameters.opt_differences != 1)) {
@@ -540,7 +568,7 @@ void args_check(std::array<bool, n_options> & used_options) {
           "must be at least 1.");
   }
 
-  if (opt_boundary < 2) {
+  if (parameters.opt_boundary < 2) {
     fatal(error_prefix, "Illegal boundary specified with -b or --boundary, "
           "must be at least 2.");
   }
@@ -567,116 +595,52 @@ void args_check(std::array<bool, n_options> & used_options) {
   }
 
   if (parameters.opt_version) {
-    show(header_message);
+    show(header_message, parameters.logfile);
     std::exit(EXIT_SUCCESS);
   }
 
   if (parameters.opt_help) {
-    show(header_message);
-    show(args_usage_message);
+    show(header_message, parameters.logfile);
+    show(args_usage_message, parameters.logfile);
     std::exit(EXIT_SUCCESS);
   }
 
   // scoring system check
-  const int64_t diff_saturation_16 = (std::min((UINT16_MAX / parameters.penalty_mismatch),
-                                               (UINT16_MAX - penalty_gapopen)
-                                               / penalty_gapextend));
+  const int64_t diff_saturation_16 = (std::min((uint16_max / parameters.penalty_mismatch),
+                                               (uint16_max - parameters.penalty_gapopen)
+                                               / parameters.penalty_gapextend));
 
   if (parameters.opt_differences > diff_saturation_16) {
     fatal(error_prefix, "Resolution (d) too high for the given scoring system.");
   }
 
-  if (parameters.penalty_mismatch > UINT8_MAX) {
+  if (parameters.penalty_mismatch > uint8_max) {
     fatal(error_prefix, "Alignment scoring system yielded a mismatch penalty greater than 255, "
           "please use different parameter values.");
   }
 }
 
 
-void open_files()
-{
-  // special case (always '-')??
-  outfile = fopen_output(parameters.opt_output_file.c_str());
-  if (outfile == nullptr) {
-    fatal(error_prefix, "Unable to open output file for writing.");
-  }
-
-  /* open files */
-
-  if (not opt_log.empty())
-    {
-      logfile = fopen_output(opt_log.c_str());
-      if (logfile == nullptr) {
-        fatal(error_prefix, "Unable to open log file for writing.");
-      }
-    }
-
-  if (not parameters.opt_seeds.empty())
-    {
-      fp_seeds = fopen_output(parameters.opt_seeds.c_str());
-      if (fp_seeds == nullptr) {
-        fatal(error_prefix, "Unable to open seeds file for writing.");
-      }
-    }
-
-  if (not parameters.opt_statistics_file.empty())
-    {
-      statsfile = fopen_output(parameters.opt_statistics_file.c_str());
-      if (statsfile == nullptr) {
-        fatal(error_prefix, "Unable to open statistics file for writing.");
-      }
-    }
-
-  if (not parameters.opt_uclust_file.empty())
-    {
-      uclustfile = fopen_output(parameters.opt_uclust_file.c_str());
-      if (uclustfile == nullptr) {
-        fatal(error_prefix, "Unable to open uclust file for writing.");
-      }
-    }
-
-  if (not parameters.opt_internal_structure.empty())
-    {
-      internal_structure_file = fopen_output(parameters.opt_internal_structure.c_str());
-      if (internal_structure_file == nullptr) {
-        fatal(error_prefix, "Unable to open internal structure file for writing.");
-      }
-    }
-
-  if (not parameters.opt_network_file.empty())
-    {
-      network_file = fopen_output(parameters.opt_network_file.c_str());
-      if (network_file == nullptr) {
-        fatal(error_prefix, "Unable to open network file for writing.");
-      }
-    }
-}
-
-
-auto close_files() -> void {
-  const std::vector<std::FILE *> file_handles
-    {network_file, internal_structure_file,
-     uclustfile, statsfile, fp_seeds, outfile,
-     logfile};
-  for (auto * const file_handle : file_handles) {
-    if (file_handle != nullptr) {
-      std::fclose(file_handle);
-    }
-  }
-}
-
-
 auto main(int argc, char** argv) -> int
 {
   // initialization and checks
-  args_init(argc, argv, used_options);
-  args_check(used_options);
-  open_files();
-  show(header_message);
-  args_show();
+  struct Parameters parameters;
+  const auto used_options = args_init(argc, argv, parameters);
+  args_check(used_options, parameters);
+  open_files(parameters);
+  show(header_message, parameters.logfile);
+  args_show(parameters);
 
   // parse fasta input
-  db_read(parameters.input_filename.c_str(), parameters);
+  std::vector<char> data_v;  // refactoring: std::string fails? .data() -> const char *
+  std::vector<struct seqinfo_s> seqindex_v;
+  std::vector<uint64_t> zobrist_tab_base_v;
+  std::vector<uint64_t> zobrist_tab_byte_base_v;
+  db_read(parameters,
+          data_v,
+          seqindex_v,
+          zobrist_tab_base_v,
+          zobrist_tab_byte_base_v);
 
   // clustering
   switch (parameters.opt_differences)
@@ -690,11 +654,12 @@ auto main(int argc, char** argv) -> int
       break;
 
     default:
-      algo_run(parameters);
+      algo_run(parameters, seqindex_v);
       break;
     }
 
   // clean up
+  zobrist_exit();
   db_free();
-  close_files();
+  close_files(parameters);
 }
