@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2024 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -21,16 +21,10 @@
     PO Box 1080 Blindern, NO-0316 Oslo, Norway
 */
 
-#include <cstdint>
 #include <cstdio>  // size_t
 
 
 auto xmalloc(std::size_t size) -> void *;
 auto xrealloc(void * ptr, std::size_t size) -> void *;
 auto xfree(void * ptr) -> void;
-auto xgetline(char ** linep, std::size_t * linecapp, FILE * stream) -> ssize_t;
-auto progress_init(const char * prompt, uint64_t size) -> void;
-auto progress_update(uint64_t progress) -> void;
-auto progress_done() -> void;
-auto fopen_input(const char * filename) -> std::FILE *;
-auto fopen_output(const char * filename) -> std::FILE *;
+auto xgetline(char ** linep, std::size_t * linecapp, std::FILE * stream) -> ssize_t;
