@@ -1,7 +1,7 @@
 /*
   SWARM
 
-  Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
+  Copyright (C) 2012-2024 Torbjorn Rognes and Frederic Mahe
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU Affero General Public License as
@@ -34,9 +34,11 @@ auto compute_hashtable_size(const uint64_t sequence_count) -> uint64_t {
   // C++20: refactor with std::bit_ceil()
   static constexpr unsigned int numerator {7};
   static constexpr unsigned int denominator {10};
+  static constexpr double divider {2.0};
   static_assert(numerator != 0, "Error: will result in a divide-by-zero");
   assert(sequence_count < 6456360425798343065); // (7 * 2^63 / 10) otherwise hashtable_size > 2^63
-  return static_cast<uint64_t>(std::pow(2, std::ceil(std::log2(denominator * (sequence_count + 1) / numerator))));
+  // GCC 6 to 9: std::log2 is not a member of std! (replace with log(x) / log(2.0) for now)
+  return static_cast<uint64_t>(std::pow(2, std::ceil(std::log(denominator * (sequence_count + 1) / numerator) / std::log(divider))));
 }
 
 
