@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2024 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -22,8 +22,8 @@
 */
 
 constexpr unsigned int qgramlength {5};
-constexpr unsigned int qgramvectorbits {1U << (2 * qgramlength)};
-constexpr unsigned int qgramvectorbytes {qgramvectorbits / 8};
+constexpr unsigned int qgramvectorbits {1U << (2 * qgramlength)};  // 1,024
+constexpr unsigned int qgramvectorbytes {qgramvectorbits / 8};  // 128
 
 using qgramvector_t = unsigned char[qgramvectorbytes];
 extern qgramvector_t * qgrams;
