@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2024 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -21,21 +21,21 @@
     PO Box 1080 Blindern, NO-0316 Oslo, Norway
 */
 
-#include <cstdint>
+#include <cstdint> // uint64_t
+#include <vector>
 
-extern uint64_t * zobrist_tab_base;
-extern uint64_t * zobrist_tab_byte_base;
 
-void zobrist_init(unsigned int n);
+auto zobrist_init(unsigned int n,
+                  std::vector<uint64_t> & zobrist_tab_base_v,
+                  std::vector<uint64_t> & zobrist_tab_byte_base_v) -> void;
 
-void zobrist_exit();
+auto zobrist_exit() -> void;
 
 auto zobrist_hash(unsigned char * seq, unsigned int len) -> uint64_t;
+
 auto zobrist_hash_delete_first(unsigned char * seq, unsigned int len) -> uint64_t;
+
 auto zobrist_hash_insert_first(unsigned char * seq, unsigned int len) -> uint64_t;
 
-inline auto zobrist_value(const unsigned int pos, const unsigned char offset) -> uint64_t
-{
-  // offset can be 0, 1 or 2 (remainder of uint64 % 3)
-  return zobrist_tab_base[4 * pos + offset];
-}
+auto zobrist_value(unsigned int pos, unsigned char offset) -> uint64_t;
+
